/**********************************************************************************************

Author : Mauro Laurenti
Version : 1.0
Date : 09/03/2012
 
CopyRight 2006-2012 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) 
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).  
The software is owned by the Author, and is protected under applicable copyright laws. 
All rights are reserved. 
Any use in violation of the foregoing restrictions may subject the 
user to criminal sanctions under applicable laws (Italian or International ones), as well as to 
civil liability for the breach of the terms and conditions of this license. 
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the following 
software doesn't aim to commercial use or monetary earning of any kind.    

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT, 
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************


 This library contains all the functions that are handy for controlling 			
 an LCD with a 44780 Hitachi controller via an I2C interface made with a PCF8574							
 	
Before using the functions you have to properly initialize the I2C to 100KHz.

**********************************************************************************************/

#include "p18cxxx.h"
#include "delay.h"
#include "ctype.h"


#ifndef FLAG_LCD_44780_I2C
#define FLAG_LCD_44780_I2C

// Prototipe for itoa from ctype lib 
char *itoa (int value, char *s);

//**************************************************
// LCD constants
// Each bit is connected to the PCF8574 
//**************************************************


#define LCD_D0  0b00000001
#define LCD_D1  0b00000010
#define LCD_D2  0b00000100	
#define LCD_D3  0b00001000	
#define LCD_E   0b00010000	
#define LCD_RW  0b00100000
#define LCD_RS  0b01000000	
#define LCD_LED 0b10000000
	
//**************************************************
// PCF8574 Address
// PCF8574_ADDRESS_H refers to the enbedded address (PCF8574 model)
// PCF8574_ADDRESS_L refers to the A0, A1, A2 pins
//**************************************************	

#define PCF8574_ADDRESS_H 0x40

#define PCF8574_ADDRESS_L 0x00 

//**************************************************
//               Constant Definitions

#define LEFT  0b00000000
#define RIGHT 0b00000100

#define TURN_ON_CURSOR  0b00000010
#define TURN_OFF_CURSOR 0b00000000

#define TURN_ON_LED  0b00000000
#define TURN_OFF_LED 0b10000000

#define BLINK_ON  0b00000001
#define BLINK_OFF 0b00000000

//**************************************************



/***************************************************
*
* Description: This function generates the Enable
*              pulse  
*
* Parameters:
*
* void
* 
* Return: 
*
* void
*
***************************************************/
void Epulse (void);


/***************************************************
*
* Description: This function sends 4 bit command out 
* 					 
*
* Parameters:
*
*            data: nibble to be sent out.
*
* Return:
*
* void
*
***************************************************/
void SendCommand (unsigned char data);


/***************************************************
*
* Description: Locate the cursor at the beginning 
*              of Line 2.
*
* Parameters:
*
* void
* 
* Return:
*
* void
*
***************************************************/
void Line2LCD(void);



/***************************************************
*
* Description: Locate the cursor at home location. 
*              First line first character
*
* Parameters:
*
* void
* 
* Return:
*
* void
*
***************************************************/
void HomeLCD(void);


/***************************************************
*
* Description: This function shifts the LCD screen on 
*              the left or rigt.
*
* Parameters:
*
* shift: LEFT:  shift on the left
*        RIGHT: shift on the right
*
* number_of_shift: Specify the number of times the 
*                  shift is executed
* 
* Return:
*
* void
*
* Note:
* You must use the LEFT, RIGHT constant.
*
***************************************************/
void ShiftLCD(char shift, char number_of_shift);


/***************************************************
*
* Description: This function shifts the LCD cursor on  
*              the left or rigt.
*
* shift: LEFT:  shift on the left
*        RIGHT: shift on the right
*
* number_of_shift: Specify the number of times the 
*                  shift is executed
* Return:
*
* void
*
* Note:
* You must use the LEFT, RIGHT constant
*
***************************************************/
void ShiftCursorLCD(char shift, char number_of_shift);


/***************************************************
*
* Description: This function locates the LCD cursor on  
*              the selected line. 
*              Tested on 20x4 16x2 LCD displays.
*
* line: number of the line (1,2,3,4)
*       
*
* Return:
*
* void
*
* Note:
* It might not work with all the LCD Diplays.
*
***************************************************/
void GotoLineLCD (char line);

/***************************************************
*
* Description: This function writes a char to the 
*              LCD display.
*
*
* Parameters:
*
* value: character to be sent (e.g WriteCharLCD ('a'); )
* 
* Return:
*
* void
*
***************************************************/
void WriteCharLCD (unsigned char value);


/***************************************************
*
* Description: This function writes a const string  
*              to the LCD display.
*
* Parameters:
*
* buffer : Is a constant string  (e.g WriteStringLCD ("Hello"); ).
* 
* Return:
*
* void
*
***************************************************/
void WriteStringLCD(const rom char *buffer);



/***************************************************
*
* Description: This function writes an array of char  
*              to the LCD display, terminated with /0
*
* Parameters:
*
* buffer : It is an array of char.
* 
* Return:
*
* void
*
***************************************************/
void WriteVarLCD(unsigned char *buffer);


/***************************************************
*
* Description: This function writes an array of char  
*              to the LCD display.
*
* Parameters:
*
* value : It is an integer that must be written to the 
*         LCD diplay.
*
* numeber_of_digits: It specifies the number of shown 
*                    digit [0-5].
*                    0: Left Justified
*                    1-5: Right Justified with n digit 
* 
* Return:
*
* void
*
* Note:
* If you set a number of digit less than required,
* the digit will be lost starting from the less
* significant digit. Minus is like a digit.
*
***************************************************/
void WriteIntLCD(int value, char number_of_digits);


/***************************************************
*
* Description: This Function cleans the LCD display
*             
*
* Parameters:
*
* void
* 
* Return:
*
* void
*
***************************************************/

void ClearLCD (void);



/***************************************************
*
* Description: This function controls the cursor
*              option (blinking, active)
*
* Parameters:
*
* active: activate the cursor, showing it
*         TURN_ON_CURSOR : cursor is ON
*         TURN_OFF_CURSOR : cursor is OFF
*
* blinking: let the cursor blink
*           BLINK_ON  : Blinking is ON
*           BLINK_OFF : Blinking is OFF   
* 
* Return:
*
* void
*
* Note:
* You must use the constants TURN_ON_CURSOR, TURN_OFF_CURSOR,
* BLINK_ON, BLINK_OFF
*
***************************************************/
void CursorLCD(char active,char blinking);


/***************************************************
*
* Description: This function controls the back ligth
*              LED 
*
* Parameters:
*
* active: activate the backlight LED.
*         TURN_ON_LED  : Backlight is ON
*         TURN_OFF_LED : Backlight is OFF
* 
* Return:
*
* void
*
* Note:
* You must use the constant TURN_ON_LED, TURN_OFF_LED
*
***************************************************/
void BacklightLCD(char active);


/***************************************************
*
* Description: This funnction initializes the LCD  
*              to work in 4 bit modality.
*
* Parameters:
*
* quartz_frequency: Quartz freq. expressed in MHz
*                   used to run the PIC.
* 
* Return:
*
* void
*
* Note:
* You must initialize the I2C module before opening the LCD driven with the PCF8574
*
***************************************************/
void  OpenLCD(unsigned char quartz_frequency);


#endif
