//*************************************************************************************************//
// Titolo: 			 			main.c (progetto telecomando domotino)
//					
// Autore:			 			Marcello Pinna	23/10/2013
// 
// Descrizione: 	 			Firmware per il controllo remoto del robot Domotino tramite moduli Xbee
//								e protocollo USART
// 
// Hardware sperimentale:		Freedom II (by Mauro Laurenti - www.Laurtec.com)
//					 			pic 18F4550
//					 			quarzo 20 MHz
//								interfaccia I2C corredata di display LCD 16x2 (vedi tutorial su www.Laurtec.com)
//								modulo Xbee (n 2) (vedi tutorial su www.settorezero.com)
//								scheda seriale DTE per Xbee (n2)
//
// Ambiente di programmazione:	MPLAB 8.40 - compilatore C18 3.40
//
// Note:						Allo scopo di garantire la portabilit, nel caso di ambienti di programmazione diversi,
// 								tutti i file di cui necessita il programma sono stati inclusi nella cartella del progetto.
//
// Fonti:						www.Laurtec.com (Mauro Laurenti)
//								www.Settorezero.com (Giovanni Bernardo)
//								www.robot-italy.com
//								www.digi.com
//*************************************************************************************************//

//************************************************************
// Sezione DEFINE
//************************************************************	
// numero di byte da inviare all'usart
#define Byte_to_Send 8

//************************************************************
// Sezione INCLUDE
//************************************************************	
#include "p18f4550.h"
#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"
#include "PCF8574.c"
#include "delay.c"
#include "usart.h"
#include "portb.h"

//************************************************************
// Sezione CONFIGURATION BITS
//************************************************************	
#pragma config FOSC = HS		//OSC = HS  	Impostato per lavorare ad alta frequenza 
#pragma config WDT = OFF		//WDT = OFF		Disabilito il watchdog timer
#pragma config LVP = OFF		//LVP = OFF 	Disabilito programmazione LVP
#pragma config PBADEN = OFF		//PBADEN = OFF 	Disabilito gli ingressi analogici

//************************************************************
// Sezione VARIABILI
//************************************************************	
// Contatore per l'invio dei byte all'usart
unsigned char i = 0; 

// Definizione dell'array per il comando da impartire al robot
unsigned char Send_Byte [8];

//************************************************************
// Routine REMOTE CONTROL (inizializzazione)
//************************************************************
// Questa routine invia il comando "M6000000" per impostare domotino in modalit REMOTE CONTROL
// (viene eseguita una volta sola, automaticamente, all'accensione oppure premendo il tasto RESET)
void Remote_Control_Init (void) {

	// filtro antirimbalzo, un secondo di ritardo
	delay_ms(1000);
	// Dichiaro l'array		
	Send_Byte[0] = 'M';
	Send_Byte[1] = '6';
	Send_Byte[2] = '0';
	Send_Byte[3] = '0';
	Send_Byte[4] = '0';
	Send_Byte[5] = '0';
	Send_Byte[6] = '0';
	Send_Byte[7] = '0';
	// Inizializzo il mio Array
	for (i = 0; i < Byte_to_Send; i++) {  
	// Invio in successione gli otto byte all'usart							
	WriteUSART (Send_Byte[i]);
	// Attendo che ogni singolo byte venga trasmesso
	while (BusyUSART());
	}
	// Pulisco il display
	ClearLCD ();
	// Invio la stringa all'LCD
	WriteStringLCD ("    Domotino   ");
	GotoLineLCD (2);
	WriteStringLCD (" Remote Control");
	}

//************************************************************
// ROUTINE PRINCIPALE
//************************************************************		
void main (void){

	// Imposto PORTA, PORTB, PORTC, PORTD e PORTE come ingressi (0xFF)
	LATA = 0x00; 
 	TRISA = 0xFF;
 	LATB = 0x00; 
 	TRISB = 0xFF;
 	LATC = 0x00;    	 
 	TRISC = 0xFF;
 	LATD = 0x00;  
 	TRISD = 0xFF;
 	LATE = 0x00;
 	TRISE = 0xFF;
	
 	// Abilito i resistori di pull-up sulla PORTB
 	EnablePullups(); 
	
 	// Inizializzo il modulo I2C a 100KHz @20MHz
	OpenI2C(MASTER, SLEW_OFF);
	SSPADD = 49;
	
	// Inizializzo il display LCD con quarzo a 20MHz
 	OpenLCD (20);
 	BacklightLCD (TURN_ON_LED);
	
	// Configuro l'USART: 8 bit, 19200 bit/s, 1 bit stop, 0 bit parit, nessuna interruzione (polling)
 	OpenUSART( USART_TX_INT_OFF &
 	USART_RX_INT_OFF &
 	USART_ASYNCH_MODE &
 	USART_EIGHT_BIT &
 	USART_CONT_RX &
 	USART_BRGH_HIGH,
 	64 );
			
	// Inizializzo il robot in modalit Remote Control
	Remote_Control_Init ();
	
	// Piccola pausa per stabilizzare il sistema
	delay_ms (1000);
	
	// Riscrivo la seconda cella dell'array, contenente il carattere "6", con il carattere "0",
	// in quanto la velocit del robot che non pu essere superiore al carattere "5" 
	WriteUSART (Send_Byte[1] = '0');
	while (BusyUSART());
		
 	while(1) {
	
		// NOTE SUI COMANDI DA IMPARTIRE A DOMOTINO PER IL SUO SPOSTAMENTO:
		// Ogni comando da inviare a domotino deve essere composto sempre da otto byte, inviati come caratteri e non come interi. 
		// I byte contenuti nella 1a. e 2a. cella del vettore corrispondono rispettivamente alla direzione ed alla velocit, gli
		// altri restano invariati; es. nel caso F100000 si ha:  F = direzione Forward, 1 = 700, ovvero la velocit minima. 
		// La lettera "v", presente nella lista dei comandi, rappresenta il valore della velocit, espressa con una scala da 0 a 5, 
		// corrispondente al segnale pwm applicato ai motori. (vedi progetto Domotino su www.Lauterc.com)
		
		// LISTA DEI COMANDI:
		// M6000000: modalit remote control (impostazione predefinita)
		// Fv000000: avanti 
		// Bv000000: indietro 
		// Lv000000: sinistra 
		// Rv000000: destra 
		// S0000000: stop 
		
		// TABELLA DELLE VELOCITA':
		// 	   0 = 0
		// v = 1 = 700
		// v = 2 = 750
		// v = 3 = 800
		// v = 4 = 850
		// v = 5 = 1023
	
		//************************************************************
		// Routine FORWARD
		//************************************************************	
		// Controllo la pressione di RB4
 		if (PORTBbits.RB4 == 0) {
		delay_ms(300);
		Send_Byte[0] = 'F';
		Send_Byte[1] = Send_Byte[1];
		Send_Byte[2] = '0';
		Send_Byte[3] = '0';
		Send_Byte[4] = '0';
		Send_Byte[5] = '0';
		Send_Byte[6] = '0';
		Send_Byte[7] = '0';
		for (i = 0; i < Byte_to_Send; i++) {  
		WriteUSART (Send_Byte[i]);
		while (BusyUSART());
		}
		ClearLCD ();
		WriteStringLCD ("Cmd  :  ");
		ShiftCursorLCD (1,1);
		WriteStringLCD ("Forward");
		GotoLineLCD (2);
		WriteStringLCD ("Speed:  ");
		ShiftCursorLCD (1,1);
		WriteCharLCD (Send_Byte[1]);
		}
		
		//************************************************************
		// Routine BACK
		//************************************************************			
		if (PORTBbits.RB5 == 0) {
		delay_ms(300);
		Send_Byte[0] = 'B';
		Send_Byte[1] = Send_Byte[1];
		Send_Byte[2] = '0';
		Send_Byte[3] = '0';
		Send_Byte[4] = '0';
		Send_Byte[5] = '0';
		Send_Byte[6] = '0';
		Send_Byte[7] = '0';
		for (i = 0; i < Byte_to_Send; i++) {  
		WriteUSART (Send_Byte[i]);
		while (BusyUSART());
		}
	 	ClearLCD ();
		WriteStringLCD ("Cmd  :  ");
		ShiftCursorLCD (1,1);
		WriteStringLCD ("Back");
		GotoLineLCD (2);
		WriteStringLCD ("Speed:  ");
		ShiftCursorLCD (1,1);
		WriteCharLCD (Send_Byte[1]);
		}
		
		//************************************************************
		// Routine LEFT
		//************************************************************			
		if (PORTBbits.RB6 == 0) {
		delay_ms(300);
		Send_Byte[0] = 'L';
		Send_Byte[1] = Send_Byte[1];
		Send_Byte[2] = '0';
		Send_Byte[3] = '0';
		Send_Byte[4] = '0';
		Send_Byte[5] = '0';
		Send_Byte[6] = '0';
		Send_Byte[7] = '0';
		for (i = 0; i < Byte_to_Send; i++) {  
		WriteUSART (Send_Byte[i]);
		while (BusyUSART());
		}
	 	ClearLCD ();
		WriteStringLCD ("Cmd  :  ");
		ShiftCursorLCD (1,1);
		WriteStringLCD ("Left");
		GotoLineLCD (2);
		WriteStringLCD ("Speed:  ");
		ShiftCursorLCD (1,1);
		WriteCharLCD (Send_Byte[1]);
		}
		
		//************************************************************
		// Routine RIGTH
		//************************************************************				
		if (PORTBbits.RB7 == 0) {
		delay_ms(300);
		Send_Byte[0] = 'R';
		Send_Byte[1] = Send_Byte[1];
		Send_Byte[2] = '0';
		Send_Byte[3] = '0';
		Send_Byte[4] = '0';
		Send_Byte[5] = '0';
		Send_Byte[6] = '0';
		Send_Byte[7] = '0';
		for (i = 0; i < Byte_to_Send; i++) {  
		WriteUSART (Send_Byte[i]);
		while (BusyUSART());
		}
	 	ClearLCD ();
		WriteStringLCD ("Cmd  :  ");
		ShiftCursorLCD (1,1);
		WriteStringLCD ("Right");
		GotoLineLCD (2);
		WriteStringLCD ("Speed:  ");
		ShiftCursorLCD (1,1);
		WriteCharLCD (Send_Byte[1]);
		}
		
		//************************************************************
		// Routine STOP
		//************************************************************				
 		if (PORTDbits.RD5 == 0) {
		delay_ms(300);
		Send_Byte[0] = 'S';
		Send_Byte[1] = '0';
		Send_Byte[2] = '0';
		Send_Byte[3] = '0';
		Send_Byte[4] = '0';
		Send_Byte[5] = '0';
		Send_Byte[6] = '0';
		Send_Byte[7] = '0';
		for (i = 0; i < Byte_to_Send; i++) {  
		WriteUSART (Send_Byte[i]);
		while (BusyUSART());
		}
	 	ClearLCD ();
		WriteStringLCD ("Cmd  :  ");
		ShiftCursorLCD (1,1);
		WriteStringLCD ("Stop");
		GotoLineLCD (2);
		WriteStringLCD ("Speed:  ");
		ShiftCursorLCD (1,1);
		WriteCharLCD (Send_Byte[1]);
		}
		
		//************************************************************
		// Routine SPEED DECREASING 
		//************************************************************		
		if (PORTDbits.RD6 == 0) {
		delay_ms(300);
		if (Send_Byte[1] > '0') {
		Send_Byte[0] = Send_Byte[0];
		Send_Byte[1] = Send_Byte[1] - 1;
		Send_Byte[2] = '0';
		Send_Byte[3] = '0';
		Send_Byte[4] = '0';
		Send_Byte[5] = '0';
		Send_Byte[6] = '0';
		Send_Byte[7] = '0';
		for (i = 0; i < Byte_to_Send; i++) {  
		WriteUSART (Send_Byte[i]);
		while (BusyUSART());
		}
	 	GotoLineLCD (2);
		WriteStringLCD ("Speed:  ");
		ShiftCursorLCD (1,1);
		WriteCharLCD (Send_Byte[1]);
		}
		else 
		Send_Byte[1] = '0';
		}
		
		//************************************************************
		// Routine SPEED INCREASING 
		//************************************************************		
		if (PORTDbits.RD7 == 0) {
		delay_ms(300);
		if (Send_Byte[1] < '5') {
		Send_Byte[0] = Send_Byte[0];
		Send_Byte[1] = Send_Byte[1] + 1;
		Send_Byte[2] = '0';
		Send_Byte[3] = '0';
		Send_Byte[4] = '0';
		Send_Byte[5] = '0';
		Send_Byte[6] = '0';
		Send_Byte[7] = '0';
		for (i = 0; i < Byte_to_Send; i++) {  
		WriteUSART (Send_Byte[i]);
		while (BusyUSART());
		}
	 	GotoLineLCD (2);
		WriteStringLCD ("Speed:  ");
		ShiftCursorLCD (1,1);
		WriteCharLCD (Send_Byte[1]);
		ShiftCursorLCD (1,0);
		WriteStringLCD ("        ");
		}
		else 
		Send_Byte[1] = '5';
		}
 	}
}

